#include <QMimeData>
#include <QDebug>
#include <QDir>


#include "glwidget.h"

Scene *s_ref = NULL;
Scene *scn = NULL;

static void init_lang(void);
void scene_clone_gl(Object *ol1, Object *ol2);
static bool load_scene_file(const char *name);


GLWidget::GLWidget()
  : QOpenGLWidget()
{
  setAcceptDrops(true);
  startTimer( 70 );
  lastfile = QString("");
  init_lang();
}

void GLWidget::initializeGL()
{
    //initialization of OpenGL
    glShadeModel( GL_SMOOTH );
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
    glEnable( GL_TEXTURE_2D );
    glEnable( GL_CULL_FACE );
    glEnable( GL_DEPTH_TEST );

    //initialization of GLEW from s3d
    init_gl(NULL);
}

void GLWidget::paintGL()
{
    if (scn == NULL || s_ref == NULL) {
       glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    } else {
       display_gl(scn);
       scene_free(scn);
       scn = NULL;
    }
}

QSize GLWidget::sizeHint() const {
    return QSize(640, 480);
}

void GLWidget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
}


void GLWidget::timerEvent(QTimerEvent *event)
{
    if (s_ref != NULL) {
        scn = scene_eval();
        scene_clone_gl(scn->objs, s_ref->objs);
        time_incr(1);
    }
    update();
}

void GLWidget::dragEnterEvent(QDragEnterEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dragMoveEvent(QDragMoveEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dropEvent(QDropEvent *event) {
    const QMimeData *mimeData = event->mimeData();
    if (mimeData->hasUrls()) {
        QList<QUrl> urlList = mimeData->urls();
        QString filename;
        QFileInfo info;
        for (int i = 0; i < urlList.size(); ++i) {
            filename = urlList.at(i).toLocalFile();
            info.setFile(filename);
            if (info.isFile()) {
                QString ext = info.suffix();
                if (ext == "scn") {
                    QDir::setCurrent(info.absolutePath());
                    if (load_scene_file(filename.toLatin1().data()))
                        event->acceptProposedAction();
                    else
                        qDebug() << "No scene";
                    lastfile = filename.toLatin1().data();
                }
             } else
                 qDebug() << "Unable to load file";
             update();
          }
      }
}

void GLWidget::dragLeaveEvent(QDragLeaveEvent *event) {
    event->accept();
}

void GLWidget::keyPressEvent(QKeyEvent *event) {
    switch (event->key()) {
    case Qt::Key_Q:
    case Qt::Key_Escape:
           exit(0);
           break;
    case Qt::Key_R:
        if (!lastfile.isEmpty())
            load_scene_file(lastfile.toLatin1().data());
        break;
    }
}


// -------------------------------------------------


static bool load_scene_file(const char *name) {
    FILE *fp = fopen(name, "r");
    if (!fp) return false;
    if (s_ref != NULL) {
        scene_free(s_ref);
    }
    if (scn != NULL) {
        close_gl(scn);
        scene_free(scn);
    }
    lang_file(fp);
    if ((s_ref = scene_read()) == NULL)
        return false;
    fclose(fp);

    load_objects_gl(s_ref->objs);
    setup_programs_gl(s_ref, s_ref->objs);

    return true;
}

void scene_clone_gl(Object *ol1, Object *ol2)
{
  Object *o1, *o2;

  for (o1 = ol1, o2 = ol2; o1 != NULL || o2 != NULL; o1 = o1->next, o2 = o2->next) {
    if (o1->type == V_GROUP && o2->type == V_GROUP) {
      scene_clone_gl(children_(o1), children_(o2));
    } else {
      if (o1->u.sh->vb == NULL)
        o1->u.sh->vb = NEWSTRUCT(GLvbuf);
      *(o1->u.sh->vb) = *(o2->u.sh->vb);
      if (o1->mat)
        o1->mat->shd.id = o2->mat->shd.id;
      if (o2->mat->tinfo)
        *((TmapInfo *)(o1->mat->tinfo)) = *((TmapInfo *)(o2->mat->tinfo));
    }
  }
}


void init_lang(void)
{
  lang_defun((char *)("scene"), scene_parse);
  lang_defun((char *)("view"), view_parse);
  lang_defun((char *)("dist_light"), distlight_parse);
  lang_defun((char *)("plastic"), plastic_parse);

  lang_defun((char *)("shaderprog"), shaderprog_parse);

  lang_defun((char *)("meshobj"), obj_parse);
  lang_defun((char *)("mesh"), imesh_parse);

  lang_defun((char *)("primobj"), obj_parse);
  lang_defun((char *)("sphere"), sphere_parse);

  lang_defun((char *)("group"), graph_parse);
  lang_defun((char *)("affine"), xaffine_parse);

  lang_defun((char *)("slideproj"), slideproj_parse);
  lang_defun((char *)("shiny"), shiny_parse);
  lang_defun((char *)("rough"), rough_parse);
  lang_defun((char *)("textured"), textured_parse);
  lang_defun((char *)("imagemap"), imagemap_parse);
  lang_defun((char *)("chequer"), chequer_parse);

  lang_defun((char *)("arg"), arg_parse);
  lang_defun((char *)("motor"), motor_parse);
}
