#include <QMimeData>
#include <QDebug>
#include <QDir>


#include "glwidget.h"

Scene *scn = NULL;

static void init_lang(void);
static bool load_scene_file(const char *name);


GLWidget::GLWidget()
  : QOpenGLWidget()
{
  setAcceptDrops(true);
  lastfile = QString("");
  m = m4_ident();
  init_lang();
}

void GLWidget::initializeGL()
{
    //initialization of OpenGL
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
    glEnable( GL_TEXTURE_2D );
    glEnable( GL_CULL_FACE );

    //initialization of GLEW from s3d
    init_gl(NULL);
}

void GLWidget::paintGL()
{
    if (scn == NULL) {
       glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    } else {
       display_gl(scn);
    }
}

QSize GLWidget::sizeHint() const
{
    return QSize(640, 480);
}

void GLWidget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
}

void GLWidget::mousePressEvent(QMouseEvent *event)
{
  if (event->button() == Qt::LeftButton) {
      m = m4_rotate('x', 0.081);
  } else if (event->button() == Qt::RightButton) {
      m = m4_rotate('x', -0.081);
  }
  if (scn != NULL) {
      scn->view->V = m4_m4prod(scn->view->V, m);
  }
  update();
}

void GLWidget::dragEnterEvent(QDragEnterEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dragMoveEvent(QDragMoveEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dropEvent(QDropEvent *event) {
    const QMimeData *mimeData = event->mimeData();
    if (mimeData->hasUrls()) {
        QList<QUrl> urlList = mimeData->urls();
        QString filename;
        QFileInfo info;
        for (int i = 0; i < urlList.size(); ++i) {
            filename = urlList.at(i).toLocalFile();
            info.setFile(filename);
            if (info.isFile()) {
                QString ext = info.suffix();
                if (ext == "scn") {
                    QDir::setCurrent(info.absolutePath());
                    if (load_scene_file(filename.toLatin1().data()))
                        event->acceptProposedAction();
                    else
                        qDebug() << "No scene";
                    lastfile = filename.toLatin1().data();
                }
             } else
                 qDebug() << "Unable to load file";
             update();
          }
      }
}

void GLWidget::dragLeaveEvent(QDragLeaveEvent *event) {
    event->accept();
}

void GLWidget::keyPressEvent(QKeyEvent *event) {
    switch (event->key()) {
    case Qt::Key_Q:
    case Qt::Key_Escape:
           exit(0);
           break;
    case Qt::Key_R:
        if (!lastfile.isEmpty())
            load_scene_file(lastfile.toLatin1().data());
        update();
        break;
    }
}


// -------------------------------------------------

extern "C" {
int st_cleanup();
}

static bool load_scene_file(const char *name) {
    FILE *fp = fopen(name, "r");
    if (!fp) return false;

    if (scn != NULL) {
        close_gl(scn);
        scene_free(scn);
        st_cleanup();
    }
    lang_file(fp);
    if ((scn = scene_read()) == NULL)
        return false;
    fclose(fp);

    load_objects_gl(scn->objs);
    setup_programs_gl(scn, scn->objs);

    return true;
}


void init_lang(void)
{
  lang_defun((char *)("scene"), scene_parse);
  lang_defun((char *)("view"), view_parse);
  lang_defun((char *)("dist_light"), distlight_parse);
  lang_defun((char *)("plastic"), plastic_parse);

  lang_defun((char *)("shaderprog"), shaderprog_parse);

  lang_defun((char *)("meshobj"), obj_parse);
  lang_defun((char *)("mesh"), imesh_parse);

  lang_defun((char *)("primobj"), obj_parse);
  lang_defun((char *)("sphere"), sphere_parse);

  lang_defun((char *)("group"), graph_parse);
  lang_defun((char *)("affine"), xaffine_parse);

  lang_defun((char *)("slideproj"), slideproj_parse);
  lang_defun((char *)("shiny"), shiny_parse);
  lang_defun((char *)("rough"), rough_parse);
  lang_defun((char *)("textured"), textured_parse);
  lang_defun((char *)("imagemap"), imagemap_parse);
  lang_defun((char *)("chequer"), chequer_parse);

  lang_defun((char *)("arg"), arg_parse);
  lang_defun((char *)("motor"), motor_parse);
}
