#include <QMimeData>
#include <QDebug>
#include <QDir>

#include "glwidget.h"

Scene *s1 = NULL;

static int w_mouse_x;
static int w_mouse_y;
static double w_width = 640;
static double w_height = 480;

static void init_lang(void);
static bool load_scene_file(const char *name);


GLWidget::GLWidget()
  : QOpenGLWidget()
{
  setAcceptDrops(true);
  lastfile = QString("");
  init_lang();
}

void GLWidget::initializeGL()
{
    //initialization of OpenGL
    glShadeModel( GL_SMOOTH );
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
    glEnable( GL_TEXTURE_2D );
    glEnable( GL_CULL_FACE );
    glEnable( GL_DEPTH_TEST );

    //initialization of GLEW from s3d
    init_gl(NULL);
}

void GLWidget::paintGL()
{
    if (s1 == NULL) {
       glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    } else {
       display_gl(s1);
    }
}

QSize GLWidget::sizeHint() const {
    return QSize(w_width, w_height);
}

void GLWidget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    w_width = width; w_height = height;
}

void GLWidget::mouseMoveEvent(QMouseEvent *event)
{
    w_mouse_x = (event->x());
    w_mouse_y = (event->y());
    update();
}

void GLWidget::dragEnterEvent(QDragEnterEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dragMoveEvent(QDragMoveEvent *event) {
    event->acceptProposedAction();
}

void GLWidget::dropEvent(QDropEvent *event) {
    const QMimeData *mimeData = event->mimeData();
    if (mimeData->hasUrls()) {
        QList<QUrl> urlList = mimeData->urls();
        QString filename;
        QFileInfo info;
        for (int i = 0; i < urlList.size(); ++i) {
            filename = urlList.at(i).toLocalFile();
            info.setFile(filename);
            if (info.isFile()) {
                QString ext = info.suffix();
                if (ext == "scn") {
                    QDir::setCurrent(info.absolutePath());
                    if (load_scene_file(filename.toLatin1().data()))
                        event->acceptProposedAction();
                    else
                        qDebug() << "No scene";
                    lastfile = filename.toLatin1().data();
                }
             } else
                 qDebug() << "Unable to load file";
             update();
          }
      }
}

void GLWidget::dragLeaveEvent(QDragLeaveEvent *event) {
    event->accept();
}

void GLWidget::keyPressEvent(QKeyEvent *event) {
    switch (event->key()) {
    case Qt::Key_Q:
    case Qt::Key_Escape:
           exit(0);
           break;
    case Qt::Key_R:
        if (!lastfile.isEmpty())
            load_scene_file(lastfile.toLatin1().data());
        break;
    }
}


// -------------------------------------------------


static bool load_scene_file(const char *name) {
    FILE *fp = fopen(name, "r");
    if (!fp) return false;

    if (s1 != NULL) {
        close_gl(s1);
        scene_free(s1);
    }
    lang_file(fp);
    if ((s1 = scene_read()) == NULL)
        return false;
    fclose(fp);

    load_objects_gl(s1->objs);
    setup_programs_gl(s1, s1->objs);

    return true;
}

void my_xform_cb(double *p, int n, void *info)
{
  int i;
  double mouse_x = (w_mouse_x/w_width) * 6.28;
  double mouse_y = (w_mouse_y/w_height) * 6.28;

  int *assgn = (int*)info;
  for (i=0; i < n; i++) {
    switch (assgn[i]) {
    case 0: p[i] = mouse_x; break;
    case 1: p[i] = mouse_y; break;
    default: break;
    }
  }
}

Val xmouse_cb_parse(int pass, Pval *pl)
{
  Val v = {V_NULL, {0}};
  if (pass == T_EXEC) { Pval *p;
    Transform *t = NULL;
    int i, *info = NULL;
    double par[9] = {0,0,0, 0,0,0, 1,1,1};
    double assgn[9] = {-1, -1, -1, -1, -1, -1, -1, -1, -1};
    for (p = pl; p != NULL; p = p->next) {
      if (p->name && strcmp(p->name, "assign") == 0 && p->val.type == V_PVL ) {
    pvl_to_array((Pval *)p->val.u.v, assgn, 9);
    info = (int *)emalloc(sizeof(int) * 9);
    for (i=0; i<9; i++)
      info[i] = assgn[i];
      } else
    error("(xparam) wrong argument");
    }
    t = xform_new(XFORM_PARAM);
    xform_setpar(t, par);
    xform_updatepar(t);
    xform_applypar_m(t);
    xform_applypar_mi(t);
    t->p_cb = my_xform_cb;
    t->info = info;
    v.u.v = t;
    v.type = V_TRANSFORM;
  }
  return v;
}

void init_lang(void)
{
  lang_defun((char *)("scene"), scene_parse);
  lang_defun((char *)("view"), view_parse);
  lang_defun((char *)("dist_light"), distlight_parse);
  lang_defun((char *)("plastic"), plastic_parse);

  lang_defun((char *)("shaderprog"), shaderprog_parse);

  lang_defun((char *)("meshobj"), obj_parse);
  lang_defun((char *)("mesh"), imesh_parse);

  lang_defun((char *)("primobj"), obj_parse);
  lang_defun((char *)("sphere"), sphere_parse);

  lang_defun((char *)("group"), graph_parse);
  lang_defun((char *)("affine"), xaffine_parse);

  lang_defun((char *)("slideproj"), slideproj_parse);
  lang_defun((char *)("shiny"), shiny_parse);
  lang_defun((char *)("rough"), rough_parse);
  lang_defun((char *)("textured"), textured_parse);
  lang_defun((char *)("imagemap"), imagemap_parse);
  lang_defun((char *)("chequer"), chequer_parse);

  lang_defun("param", xparam_parse);
  lang_defun("mouse", xmouse_cb_parse);

  lang_defun((char *)("arg"), arg_parse);
  lang_defun((char *)("motor"), motor_parse);
}
